package org.colinvella.fancyfish.block;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.creativetab.ModCreativeTabs;
import org.colinvella.fancyfish.gui.FishTankGuiHandler;
import org.colinvella.fancyfish.tileentity.FishTankTileEntity;

import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentTranslation;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumWorldBlockLayer;
import net.minecraft.world.World;

public class FishTankBlock extends ModContainerBlock
{
	public static final String ID = "FishTank";
	
	// properties mapped to meta data
	public static final PropertyBool PROPERTY_FILLED = PropertyBool.create("filled");
	  
	public FishTankBlock()
	{
		super(ID, Material.sand, ModCreativeTabs.FancyFishTab);
	}
	
	// override: return false as it is an irregular block
	@Override
	public boolean isOpaqueCube()
	{
		return false;
	}
	
	// override: treat block as translucent due to glass and water
	@Override
	public EnumWorldBlockLayer getBlockLayer()
	{
		return EnumWorldBlockLayer.TRANSLUCENT;
	}
	
	// getStateFromMeta, getMetaFromState are used to inter-convert between the block's property values and
	//   the stored metadata (which must be an integer in the range 0 - 15 inclusive)
	@Override
	public IBlockState getStateFromMeta(int meta)
	{
		boolean filled = meta > 0;
		return this.getDefaultState().withProperty(PROPERTY_FILLED, filled);
	}
	
	@Override
	public int getMetaFromState(IBlockState state)
	{
		boolean filled = (Boolean)state.getValue(PROPERTY_FILLED);	
		return filled ? 1 : 0;
	}

	// tile entity for storing content
	@Override
	public TileEntity createNewTileEntity(World worldIn, int meta)
	{
		return new FishTankTileEntity();
	}
	
	// necessary to define which properties your blocks use
	// will also affect the variants listed in the block states model file
	@Override
	protected BlockState createBlockState()
	{
	    return new BlockState(this, new IProperty[] {PROPERTY_FILLED});
	}

	@Override
	public boolean onBlockActivated(World world, BlockPos pos, IBlockState state,
			EntityPlayer player, EnumFacing side, float hitX, float hitY, float hitZ)
	{
		if (world.isRemote)
			return super.onBlockActivated(world, pos, state, player, side, hitX, hitY, hitZ);
			
		// do nothing if with a bucket in hand
		ItemStack currentItemStack = player.getCurrentEquippedItem();
		if (currentItemStack != null)
		{
			Item currentItem = currentItemStack.getItem();
			if (currentItem == Items.bucket || currentItem == Items.water_bucket)
				return super.onBlockActivated(world, pos, state, player, side, hitX, hitY, hitZ);
		}
		
		if (state != state.withProperty(FishTankBlock.PROPERTY_FILLED, true))
		{
			player.addChatMessage(new ChatComponentTranslation("tile.FishTank.message.needsWater"));
		}
		else
		{
	        player.openGui(FancyFishMod.instance, FishTankGuiHandler.MOD_TILE_ENTITY_GUI, world, pos.getX(), pos.getY(), pos.getZ());
		}
		
	    return true;
	}
	
	@Override
	public void breakBlock(World world, BlockPos pos, IBlockState blockState)
	{
    	// get tile entity
        FishTankTileEntity fishTankTileEntity = (FishTankTileEntity) world.getTileEntity(pos);
        
        // eject any contained fish
        fishTankTileEntity.ejectFishEntities();
        
		super.breakBlock(world, pos, blockState);
	}

}
