package org.colinvella.fancyfish.entity.ai;

import org.colinvella.fancyfish.entity.fish.FishEntity;

import net.minecraft.entity.EntityCreature;
import net.minecraft.entity.ai.EntityAIBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.pathfinding.PathNavigateGround;
import net.minecraft.util.Vec3;

public class FishEntityAITempt extends EntityAIBase
{
    public FishEntityAITempt(FishEntity temptedFishEntity, Item temptItem, boolean scaredByPlayerMovement)
    {
        this.temptedFishEntity = temptedFishEntity;
        this.temptItem = temptItem;
        this.scaredByPlayerMovement = scaredByPlayerMovement;
        this.setMutexBits(1);
    }

    /**
     * Returns whether the EntityAIBase should begin execution.
     */
    public boolean shouldExecute()
    {
        if (this.delayTemptCounter > 0)
        {
            --this.delayTemptCounter;
            return false;
        }
        else
        {
            this.temptingPlayer = this.temptedFishEntity.worldObj.getClosestPlayerToEntity(this.temptedFishEntity, 10.0D);

            if (this.temptingPlayer == null || !temptingPlayer.isInWater())
            {
                return false;
            }
            else
            {
                ItemStack itemstack = this.temptingPlayer.getCurrentEquippedItem();
                return itemstack == null ? false : itemstack.getItem() == this.temptItem;
            }
        }
    }

    /**
     * Returns whether an in-progress EntityAIBase should continue executing
     */
    public boolean continueExecuting()
    {
    	if (temptCounter < 0)
    		return false;
    	
        if (this.scaredByPlayerMovement)
        {
            if (distance < 6 && temptingPlayer.velocityChanged)
            {
            	return false;
            }

        }

        return distance > 0.5;
    }

    /**
     * Execute a one shot task or start executing a continuous task
     */
    public void startExecuting()
    {
    }

    /**
     * Resets the task
     */
    public void resetTask()
    {
        temptingPlayer = null;
        delayTemptCounter = 20;
        temptCounter = 50;
    }

    /**
     * Updates the task
     */
    public void updateTask()
    {
    	temptCounter--;
    	
        //update distance between tempted fish to tempting player
        distance = temptedFishEntity.getDistance(temptingPlayer.posX, temptingPlayer.posY, temptingPlayer.posZ);
        if (distance < 0.001)
        	distance = 0.001;

    	// get offset from fish to player
        double targetMotionX = temptingPlayer.posX - temptedFishEntity.posX;
        double targetMotionY = temptingPlayer.posY - temptedFishEntity.posY;
        double targetMotionZ = temptingPlayer.posZ - temptedFishEntity.posZ;
        
        targetMotionY = -1;
        
        // normalise offset to unit vector and multiply by fish's speed
        double factor = temptedFishEntity.getSwimmingSpeed() / distance;
        targetMotionX *= factor;
        targetMotionY *= factor;
        targetMotionZ *= factor;
        
        temptedFishEntity.setTargetMotion(targetMotionX, targetMotionY, targetMotionZ);
    }
    
    /** The entity using this AI that is tempted by the player. */
    private FishEntity temptedFishEntity;
   
    /** The player that is tempting the entity that is using this AI. */
    private EntityPlayer temptingPlayer;
    
    /**
     * A counter that is decremented each time the shouldExecute method is called. The shouldExecute method will always
     * return false if delayTemptCounter is greater than 0.
     */
    private int delayTemptCounter;
    private int temptCounter;
    
    private Item temptItem;
    
    // current distance between tempting player and tempted fish
    private double distance;
    
    /** Whether the entity using this AI will be scared by the tempter's sudden movement. */
    private boolean scaredByPlayerMovement;
}