package org.colinvella.fancyfish.recipe;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fml.common.registry.GameRegistry;

public abstract class ShapedRecipe extends ModRecipe
{
	public ShapedRecipe(String id, ItemStack output, String[] shape, Map<Character, ItemStack> legend)
	{
		super(id);		
		logger.info("Defining shaped recipe " + id + "...");


		if (!validateParameters(shape, legend))
			return;

		// add shape
		ArrayList<Object> parameters = new ArrayList<Object>();
		for (String shapeRow: shape)
		{
			String paddedShapeRow = String.format("%1$3s", shapeRow);
			logger.debug("[" + paddedShapeRow + "]");
			
			parameters.add(shapeRow);
		}
		
		// add legend
		logger.debug("LEGEND:");
		for (Map.Entry<Character, ItemStack> entry : legend.entrySet())
		{
			char key = entry.getKey();
			ItemStack value = entry.getValue();
			parameters.add(key);
			parameters.add(value);
			
			logger.debug(key + ": " + value.getUnlocalizedName());
		}
				
		GameRegistry.addRecipe(output, parameters.toArray());
	}

	private boolean validateParameters(String[] shape, Map<Character, ItemStack> legend)
	{
		// validate shape structure - rows
		if (shape.length < 1 || shape.length > 3)
		{
			logger.error("Shape array must consist of one, two or three rows only");
			return false;
		}
		
		// validate shape structure - columns
		boolean emptyShape = true;
		for(String shapeRow: shape)
		{
			if (shapeRow.trim().length() > 0)
				emptyShape = false;
			
			if (shapeRow.length() > 3)
			{
				logger.error("No more than three shape columns allowed");
				return false;
			}
			
			for (char cell: shapeRow.toCharArray())
			{
				if (cell != ' ' && !legend.containsKey(cell))
				{
					logger.error("Shape character '" + cell + "' not in legend");
					return false;
				}
			}
		}
		
		// validate shape structure - must be non-empty
		if (emptyShape)
		{
			logger.error("Empty shape not allowed");
			return false;			
		}	
		
		return true;
	}
	
	public static Map<Character, ItemStack> MakeStackLegend(Map<Character, Item> itemLegend)
	{
		Map<Character, ItemStack> stackLegend = new HashMap<Character, ItemStack>();
		for(Character key: itemLegend.keySet())
			stackLegend.put(key, new ItemStack(itemLegend.get(key)));
		return stackLegend;
	}
}
