package org.colinvella.fancyfish.world.gen.feature;

import java.util.Random;

import org.colinvella.fancyfish.block.ClamBlock;
import org.colinvella.fancyfish.block.ModBlockRegistry;
import org.colinvella.fancyfish.configuration.ModConfiguration;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.init.Blocks;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

public class ClamWorldGenerator extends ModWorldGenerator
{
	public static String ID = "Clam";
	
    public ClamWorldGenerator()
    {
    	super(ID);
        this.clamBlock = ModBlockRegistry.Clam;
    }
    
	@Override
	public void generate(Random random, int chunkX, int chunkZ, World world, IChunkProvider chunkGenerator,
			IChunkProvider chunkProvider) 
	{
		// do not generate features if disabled in mod config
        if (!ModConfiguration.getTerrainGeneration())
        	return;
		
		// only works for regular overworld
		if (world.provider.getDimensionId() != 0)
			return;
		
		int clamsPerChunk = 2 + random.nextInt(9);
		for (int clamIndex = 0; clamIndex < clamsPerChunk; clamIndex++)
		{
			int genX = chunkX * 16 + random.nextInt(16);
			int genZ = chunkZ * 16 + random.nextInt(16);
			int genY = world.getHeight() - 1;
			BlockPos pos = new BlockPos(genX, genY, genZ);
			Block scanBlock = null;
            while(pos.getY() > 0)
            {
            	scanBlock = world.getBlockState(pos).getBlock();
            	if (scanBlock != Blocks.air && scanBlock != Blocks.water)
            		break;
            	pos = pos.down();
            }
            
            if (scanBlock != Blocks.dirt && scanBlock != Blocks.sand && scanBlock != Blocks.clay)
            	continue;
            pos = pos.up();

            if (world.getBlockState(pos.up()).getBlock() != Blocks.water)
            	continue;
            
            boolean clamOpen = random.nextBoolean();
            EnumFacing clamFacing = EnumFacing.HORIZONTALS[random.nextInt(4)];
            
            logger.debug("Geneating clam at " + pos);
            world.setBlockState(pos,
            	this.clamBlock.getDefaultState()
            		.withProperty(ClamBlock.PROPERTY_FACING, clamFacing)
            		.withProperty(ClamBlock.PROPERTY_OPEN,  clamOpen));
		}
	}
	
    private Block clamBlock;
}
